# -*- coding: utf-8 -*-

import logging
import re
from datetime import datetime, timedelta, tzinfo


logger = logging.getLogger('yledl')


class FixedOffset(tzinfo):
    def __init__(self, offset_hours):
        self.__offset = timedelta(hours=offset_hours)

    def utcoffset(self, dt):
        return self.__offset

    def tzname(self, dt):
        return 'FixedOffset'

    def dst(self, dt):
        return timedelta(0)


def parse_areena_timestamp(timestamp):
    if timestamp is None:
        return None

    timestamp = timestamp.strip()
    parsed = parse_areena_timestamp_py3(timestamp)
    if parsed is None:
        logger.warning('Failed to parse timestamp: {}'.format(timestamp))

    return parsed


def parse_areena_timestamp_py3(timestamp):
    # Python prior to 3.7 doesn't support a colon in the timezone
    if re.search(r'\d\d:\d\d$', timestamp):
        timestamp = timestamp[:-3] + timestamp[-2:]

    return (strptime_or_none(timestamp, '%Y-%m-%dT%H:%M:%S.%f%z') or
            strptime_or_none(timestamp, '%Y-%m-%dT%H:%M:%S%z'))


def strptime_or_none(timestamp, format):
    try:
        return datetime.strptime(timestamp, format)
    except ValueError:
        return None



import pytest


def pytest_addoption(parser):
    parser.addoption('--geoblocked', action='store_true',
                     help='Enable get-blocked tests that work only in Finland')


def pytest_configure(config):
    config.addinivalue_line("markers", "geoblocked: get-blocked test that work only in Finland")


def pytest_collection_modifyitems(config, items):
    if not config.option.geoblocked:
        # Skip tests marked as geoblocked
        skip_geoblocked = pytest.mark.skip(reason="need --geoblocked option to run")
        for item in items:
            if "geoblocked" in item.keywords:
                item.add_marker(skip_geoblocked)



import json

def load_json(url, httpclient, headers=None):
    """Download and parse a JSON document at a given URL."""
    json_string = httpclient.download_page(url, headers)
    if not json_string:
        return None

    try:
        json_parsed = json.loads(json_string)
    except ValueError:
        return None

    return json_parsed


def parse_json_object(text, start_pos):
    """Extract a JSON document from a text string starting at start_pos."""
    return json.JSONDecoder().raw_decode(text[start_pos:])[0]



# -*- coding: utf-8 -*-

from utils import fetch_title, fetch_stream_url, fetch_metadata
from yledl import StreamFilters


def test_radio_title():
    title = fetch_title('https://areena.yle.fi/1-3361013')

    assert len(title) == 1
    assert title[0].startswith(
        'Tiedeykkönen: '
        'Suorat aurinkobiopolttoaineet mullistavat energiantuotannon')


def test_radio_title_hls():
    title = fetch_title('https://areena.yle.fi/1-4551633')

    assert len(title) == 1
    assert title[0].startswith(
        'Tiedeykkönen Extra: Ilmastonmuutos: '
        'Ihminen elää ilman vettä vain muutaman päivän')


def test_radio_stream_url_hls():
    url = fetch_stream_url('https://areena.yle.fi/1-4551633')

    assert len(url) == 1
    assert 'a.mp3' in url[0]


def test_radio_metadata_hls():
    metadata = fetch_metadata('https://areena.yle.fi/1-4551633')

    assert len(metadata) == 1
    assert len(metadata[0]['flavors']) == 1
    assert metadata[0]['flavors'][0]['media_type'] == 'audio'
    assert metadata[0]['duration_seconds'] == 2954
    assert len(metadata[0]['description']) > 150


def test_radio_live_url():
    url = fetch_stream_url('https://areena.yle.fi/audio/ohjelmat/57-kpDBBz8Pz')

    assert len(url) == 1
    assert '.m3u8' in url[0]


def test_radio_live_url2():
    url = fetch_stream_url(
        'https://areena.yle.fi/audio/ohjelmat/57-3gO4bl7J6?'
        '_c=yle-radio-suomi-oulu')

    assert len(url) == 1
    assert '.m3u8' in url[0]

def test_radio_live_metadata():
    metadata = fetch_metadata('https://areena.yle.fi/audio/ohjelmat/57-kpDBBz8Pz')

    assert len(metadata) == 1
    assert len(metadata[0]['flavors']) >= 1
    assert all(f.get('media_type') == 'audio' for f in metadata[0]['flavors'])
    assert metadata[0]['title'].startswith('Yle Puhe')


def test_radio_series_2020():
    urls = fetch_stream_url('https://areena.yle.fi/audio/1-50198109')

    assert len(urls) >= 6


def test_radio_metadata_2020():
    metadata = fetch_metadata('https://areena.yle.fi/audio/1-50198110')

    assert len(metadata) == 1
    assert len(metadata[0]['flavors']) == 1
    assert metadata[0]['flavors'][0]['media_type'] == 'audio'
    assert metadata[0]['duration_seconds'] == 1451
    assert len(metadata[0]['description']) > 150



# -*- coding: utf-8 -*-

import attr
import logging
import json
import base64
from .backends import HLSBackend, HLSAudioBackend, WgetBackend
from .streamflavor import StreamFlavor, FailedFlavor
from .subtitles import EmbeddedSubtitle


logger = logging.getLogger('yledl')


class KalturaApiClient(object):
    def __init__(self, api_url, httpclient):
        self.api_url = api_url
        self.httpclient = httpclient

    def start_widget_session(self, widget_id):
        return {
            'service': 'session',
            'action': 'startWidgetSession',
            'widgetId': widget_id
        }

    def list_base_entry(self, entry_id, ks):
        return {
            'service': 'baseEntry',
            'action': 'list',
            'ks': ks,
            'filter': {
                'redirectFromEntryId': entry_id
            },
            'responseProfile': {
                'fields': ('id,name,description,thumbnailUrl,dataUrl,duration,'
                           'msDuration,flavorParamsIds,mediaType,type,tags,'
                           'dvrStatus'),
                'type': 1
            }
        }

    def list_metadata(self, entry_id, ks):
        return {
            'service': 'metadata_metadata',
            'action': 'list',
            'filter': {
                'objectType': 'KalturaMetadataFilter',
                'objectIdEqual': entry_id,
                'metadataObjectTypeEqual': '1'
            },
            'ks': ks
        }

    def get_playback_context(self, entry_id, ks):
        return {
            'service': 'baseEntry',
            'action': 'getPlaybackContext',
            'entryId': entry_id,
            'ks': ks,
            'contextDataParams': {
                'objectType': 'KalturaContextDataParams',
                'flavorTags': 'all'
            }
        }

    def multi_request(self, subrequests, client_tag, partner_id):
        mrequest = {
            'apiVersion': '3.3.0',
            'format': 1,
            'ks': '',
            'clientTag': client_tag,
            'partnerId': partner_id
        }
        mrequest.update({str(i): req for i, req in enumerate(subrequests)})
        return mrequest

    def perform_request(self, request, referrer, origin):
        endpoint = self.api_url + '/api_v3/service/multirequest'
        extra_headers = {
            'Referer': referrer,
            'Origin': origin,
            'Cache-Control': 'max-age=0'
        }
        r = self.httpclient.post(endpoint, request, extra_headers)
        return r.json()


class YleKalturaApiClient(KalturaApiClient):
    partner_id = '1955031'
    widget_id = '_1955031'
    client_tag = 'html5:v0.39.4'
    api_url = 'https://cdnapisec.kaltura.com'
    http_origin = 'https://areena.yle.fi'

    def __init__(self, requests_session):
        super(YleKalturaApiClient, self).__init__(self.api_url, requests_session)

    def playback_context(self, entry_id, referrer):
        subrequests = [
            self.start_widget_session(self.widget_id),
            self.list_base_entry(entry_id, '{1:result:ks}'),
            self.get_playback_context(entry_id, '{1:result:ks}'),
            self.list_metadata(entry_id, '{1:result:ks}')
        ]

        logger.debug('Sending Kaltura API flavors request:\n' +
                     json.dumps(subrequests, indent=2))

        response = self.perform_request(
            self.multi_request(subrequests, self.client_tag, self.partner_id),
            referrer, self.http_origin)

        logger.debug('Kaltura API response:\n' +
                     json.dumps(response, indent=2))

        return response[2] if len(response) > 2 else None

    def parse_stream_flavors(self, playback_context, referrer):
        if playback_context is None:
            return [FailedFlavor('No Kaltura playback context')]

        flavor_assets = playback_context.get('flavorAssets', {})
        sources = playback_context.get('sources', {})
        delivery_profiles = self.delivery_profiles_by_flavor_id(sources)

        filtered_flavors = [fl for fl in flavor_assets
                            if self.is_web_stream(fl)]
        num_non_web = len(flavor_assets) - len(filtered_flavors)
        if num_non_web:
            logger.debug('Ignored %d non-web flavors' % num_non_web)

        return self.create_flavors(filtered_flavors, delivery_profiles, referrer)

    def parse_embedded_subtitles(self, playback_context):
        language_name_to_code = {
            'finnish': 'fin',
            'swedish': 'swe'
        }

        categories = {
            'översättning': 'käännöstekstitys',
            'programtextning': 'ohjelmatekstitys'
        }

        subtitles = []
        for caption in playback_context.get('playbackCaptions', []):
            language_name = caption.get('language', '').lower()
            language_code = language_name_to_code.get(language_name, 'unk')
            label = caption.get('label')
            category = categories.get(label, label)
            subtitles.append(EmbeddedSubtitle(language_code, category))

        return subtitles

    def create_flavors(self, flavors, delivery_profiles, referrer):
        res = []
        for flavor in flavors:
            flavor_id = flavor.get('id')
            entry_id = flavor.get('entryId')
            ext = '.' + flavor.get('fileExt', 'mp4')
            media_type = self.flavor_media_type(flavor)

            backends = []
            for profile in delivery_profiles.get(flavor_id, []):
                backends.extend(profile.backends(
                    entry_id, media_type, ext, self.partner_id,
                    self.client_tag, referrer))

            res.append(StreamFlavor(
                media_type=media_type,
                height=flavor.get('height') or None,
                width=flavor.get('width') or None,
                bitrate=flavor.get('bitrate'),
                streams=backends
            ))

        return res

    def flavor_media_type(self, flavor):
        audio_stream = ('audio_only' in self.flavor_tags(flavor) or
                        flavor.get('containerFormat') == 'mpeg audio')
        return 'audio' if audio_stream else 'video'

    def flavor_tags(self, flavor):
        tags_string = flavor.get('tags')
        return tags_string.split(',') if tags_string else []

    def is_web_stream(self, flavor):
        tags = self.flavor_tags(flavor)
        web = 'web' in tags and 'source' not in tags
        mbr = 'mbr' in tags and flavor.get('fileExt') == 'mp4'
        ipad = 'ipad' in tags or 'iphone' in tags
        return web or mbr or ipad

    def delivery_profiles_by_flavor_id(self, sources):
        format_whitelist = ['url', 'applehttp']
        valid_sources = [s for s in sources
                         if s.get('format') in format_whitelist]

        profiles_by_id_and_format = {}
        for source in valid_sources:
            flavor_ids = source.get('flavorIds', '').split(',')
            source_format = source.get('format')
            url = source.get('url')
            manifest_file = url.split('/')[-1]

            for flavor_id in flavor_ids:
                profile = DeliveryProfile(flavor_id, source_format,
                                          manifest_file)

                pkey = flavor_id + '_' + source_format
                profiles_by_id_and_format[pkey] = profile

        profiles = {}
        for p in profiles_by_id_and_format.values():
            profiles.setdefault(p.flavor_id, []).append(p)

        return profiles


@attr.s
class DeliveryProfile(object):
    flavor_id = attr.ib()
    stream_format = attr.ib()
    manifest_file = attr.ib()

    def manifest_url(self, entry_id, partner_id, client_tag, referrer):
        b64referrer = base64.b64encode(referrer.encode('utf-8')).decode('utf-8')
        return ('https://cdnsecakmi.kaltura.com/p/{partner_id}/'
                'sp/{partner_id}00/playManifest/entryId/{entry_id}/'
                'flavorId/{flavor_id}/format/{stream_format}/protocol/https/'
                '{manifest_file}?uiConfId=43362851&referrer={referrer}'
                '&playSessionId=11111111-1111-1111-1111-111111111111'
                '&clientTag={client_tag}'.format(
                    partner_id=partner_id,
                    entry_id=entry_id,
                    flavor_id=self.flavor_id,
                    stream_format=self.stream_format,
                    manifest_file=self.manifest_file,
                    referrer=b64referrer,
                    client_tag=client_tag))

    def backends(self, entry_id, media_type, file_ext, partner_id,
                 client_tag, referrer):
        backends = []
        manifest_url = self.manifest_url(entry_id, partner_id,
                                         client_tag, referrer)

        if self.stream_format == 'url':
            backends.append(WgetBackend(manifest_url, file_ext))
        elif media_type == 'video':
            backends.append(HLSBackend(manifest_url))
        else:
            backends.append(HLSAudioBackend(manifest_url))

        return backends



# -*- coding: utf-8 -*-

import pytest
from utils import fetch_title, fetch_stream_url, fetch_episode_pages, \
    fetch_metadata


def test_areena_html5_stream_url():
    streamurl = fetch_stream_url('https://areena.yle.fi/1-787136')

    assert len(streamurl) == 1
    assert '/a.m3u8?' in streamurl[0]


def test_areena_html5_metadata():
    metadata = fetch_metadata('https://areena.yle.fi/1-787136')

    assert len(metadata) == 1
    flavors = metadata[0]['flavors']
    assert len(flavors) >= 5
    assert all(f.get('media_type') == 'video' for f in flavors)
    assert all('bitrate' in f and
               'height' in f and
               'width' in f
               for f in flavors)
    assert metadata[0]['duration_seconds'] == 907
    assert metadata[0]['region'] == 'World'
    assert metadata[0]['publish_timestamp'] == '2021-04-01T00:01:00+03:00'
    assert 'expired_timestamp' not in metadata[0]
    assert len(metadata[0]['description']) > 150


def test_metadata_language():
    meta_fin = fetch_metadata(
        'https://areena.yle.fi/1-403848', meta_language='fin')
    title_fin = meta_fin[0].get('title')
    assert title_fin.startswith('Suomen tie jatkosotaan')

    meta_swe = fetch_metadata(
        'https://areena.yle.fi/1-403848', meta_language='swe')
    title_swe = meta_swe[0].get('title')
    assert title_swe.startswith('Finlands väg till fortsättningskriget')


def test_areena_series_titles():
    titles = fetch_title('https://areena.yle.fi/1-3826480')

    assert len(titles) == 10
    assert all(['Suomi on ruotsalainen' in t for t in titles])


def test_areena_series_urls():
    urls = fetch_stream_url('https://areena.yle.fi/1-3826480')

    assert len(urls) == 10
    assert all(['a.m3u8' in url for url in urls])


@pytest.mark.geoblocked
def test_areena_live_url():
    streamurl = fetch_stream_url('https://areena.yle.fi/tv/suorat/yle-tv1')

    assert len(streamurl) == 1
    assert '.m3u8' in streamurl[0]


@pytest.mark.geoblocked
def test_areena_live_metadata():
    metadata = fetch_metadata('https://areena.yle.fi/tv/suorat/yle-tv1')

    assert len(metadata) == 1
    assert len(metadata[0]['flavors']) >= 1
    assert all(f.get('media_type') == 'video' for f in metadata[0]['flavors'])
    assert metadata[0]['region'] == 'Finland'


@pytest.mark.geoblocked
def test_areena_ohjelmat_embedded_live_url():
    streamurl = fetch_stream_url('https://areena.yle.fi/tv/ohjelmat/30-595?play=yle-tv2')

    assert len(streamurl) == 1
    assert 'master.m3u8' in streamurl[0]


@pytest.mark.geoblocked
def test_areena_ohjelmat_embedded_live_metadata():
    metadata = fetch_metadata('https://areena.yle.fi/tv/ohjelmat/30-595?play=yle-tv2')

    assert len(metadata) == 1
    assert len(metadata[0]['flavors']) >= 1
    assert all(f.get('media_type') == 'video' for f in metadata[0]['flavors'])
    assert metadata[0]['region'] == 'Finland'


def test_areena_html5_clip_title():
    title = fetch_title('https://areena.yle.fi/1-3523087')

    assert len(title) == 1
    assert 'Metsien kätkemä' in title[0]


def test_areena_html5_clip_stream_url():
    streamurl = fetch_stream_url('https://areena.yle.fi/1-3523087')

    assert len(streamurl) == 1
    assert '/a.m3u8?' in streamurl[0]


def test_areena_awsmpodamdipv4_stream_url():
    streamurl = fetch_stream_url('https://areena.yle.fi/1-50875269')

    assert len(streamurl) == 1
    assert '/index.m3u8?' in streamurl[0]


def test_areena_awsmpodamdipv4_metadata():
    metadata = fetch_metadata('https://areena.yle.fi/1-50875269')

    assert len(metadata) == 1
    flavors = metadata[0]['flavors']
    assert len(flavors) >= 1
    assert all(f.get('media_type') == 'video' for f in flavors)
    assert metadata[0]['duration_seconds'] == 257
    assert metadata[0]['region'] == 'World'
    assert metadata[0]['publish_timestamp'] == '2021-06-11T08:40:00+03:00'
    assert 'expired_timestamp' not in metadata[0]
    assert len(metadata[0]['description']) > 150


def test_areena_episode_pages():
    episodes = fetch_episode_pages('https://areena.yle.fi/1-3148871')

    # The first page contains 12 episodes, make sure we get several pages
    assert len(episodes) > 20



# -*- coding: utf-8 -*-

import re
from datetime import datetime
from collections import defaultdict


class TitleFormatter(object):
    def __init__(self, template='${series}${title}${episode}${timestamp}'):
        self.template = template
        self.tokens = self._parse_template(template)

    def format(self, title, publish_timestamp=None, series_title=None,
               subheading=None, season=None, episode=None, program_id=None):
        if title is None:
            return None

        title = title.strip()
        series_title = \
            series_title.strip() if series_title is not None else None
        subheading = subheading.strip() if subheading is not None else None

        main_title = self._main_title(title, subheading, series_title)
        if not main_title and series_title:
            main_title = series_title
            series_title = None

        values = {
            'series': series_title or '',
            'title': main_title,
            'episode': self._episode_number(season, episode),
            'timestamp': self._timestamp_string(publish_timestamp),
            'date': self._date_string(publish_timestamp),
            'program_id': program_id,
        }
        separators = defaultdict(lambda: ': ',
                                 timestamp='-',
                                 date='-',
                                 program_id='-')

        return self._substitute(self.tokens, values, separators)

    def is_constant_pattern(self):
        return all(t.is_constant() for t in self.tokens)

    def _parse_template(self, template):
        res = []

        last_pos = 0
        for m in re.finditer(r'\$(:?{[a-zA-Z_]+?}|\$)', template):
            if m.start() != last_pos:
                res.append(Literal(template[last_pos:m.start()]))

            if m.group() == '$$':
                res.append(Literal('$'))
            else:
                var_name = m.group()
                res.append(Substitution(var_name))

            last_pos = m.end()

        if last_pos != len(template):
            res.append(Literal(template[last_pos:]))

        return res

    def _substitute(self, tokens, values, separators):
        res = []
        empty_separator = defaultdict(lambda: '')
        for token in self.tokens:
            sep = empty_separator if len(res) == 0 else separators
            subst = token.substitute(values, sep)
            if subst:
                res.append(subst)

        return ''.join(res)

    def _main_title(self, title, subheading, series_title):
        main_title = self._remove_genre_prefix(
            self._remove_repeated_main_title(title))
        ageless_title = self._remove_age_limit(main_title)

        if series_title:
            series_prefix = re.escape(series_title) + r'\b'
            if re.match(series_prefix, ageless_title, re.IGNORECASE):
                main_title = ageless_title[len(series_title):]
                main_title = main_title.lstrip(':').lstrip(' ')

        if subheading and not main_title:
            return subheading
        if subheading and subheading not in main_title:
            return main_title + ': ' + subheading
        else:
            return main_title

    def _remove_age_limit(self, title):
        """Strip (S) or (12) postfix from the title."""
        m = re.match(r'(.+?)\s+\(([A-Z]|[0-9]{1,2})\)$', title)
        return m.group(1) if m else title

    def _remove_repeated_main_title(self, title):
        if ':' in title:
            prefix, rest = title.split(':', 1)
            if prefix in rest:
                return rest.strip()

        return title

    def _remove_genre_prefix(self, title):
        genre_prefixes = ['Elokuva:', 'Kino:', 'Kino Klassikko:',
                          'Kino Suomi:', 'Kotikatsomo:', 'Uusi Kino:', 'Dok:',
                          'Dokumenttiprojekti:', 'Historia:']
        for prefix in genre_prefixes:
            if title.startswith(prefix):
                return title[len(prefix):].strip()
        return title

    def _episode_number(self, season, episode):
        if season and episode:
            return 'S%02dE%02d' % (season, episode)
        elif episode:
            return 'E%02d' % (episode)
        else:
            return ''

    def _timestamp_string(self, publish_timestamp):
        if publish_timestamp and hasattr(publish_timestamp, 'hour'):
            return datetime.strftime(publish_timestamp, '%Y-%m-%dT%H:%M')
        elif publish_timestamp:
            return self._date_string(publish_timestamp)
        else:
            return ''

    def _date_string(self, publish_timestamp):
        if publish_timestamp:
            return publish_timestamp.strftime('%Y-%m-%d')
        else:
            return ''


class Substitution(object):
    def __init__(self, variable_name):
        self.variable_name = variable_name

    def is_constant(self):
        return False

    def substitute(self, values, separators):
        key = self.variable_name[2:-1]
        val = values.get(key, self.variable_name)
        return separators[key] + val if val else ''


class Literal(object):
    def __init__(self, text):
        self.text = text

    def is_constant(self):
        return True

    def substitute(self, values, separators):
        return self.text



# -*- coding: utf-8 -*-

import logging
from .backends import HLSBackend
from .streamflavor import StreamFlavor, FailedFlavor


logger = logging.getLogger('yledl')


class FullHDFlavorProber(object):
    def probe_flavors(self, manifest_url, is_live, ffprobe):
        try:
            programs = ffprobe.show_programs_for_url(manifest_url)
        except ValueError as ex:
            return [FailedFlavor('Failed to probe stream: {}'.format(str(ex)))]

        return self.programs_to_stream_flavors(programs, manifest_url, is_live)

    def programs_to_stream_flavors(self, programs, manifest_url, is_live):
        res = []
        for program in programs.get('programs', []):
            streams = program.get('streams', [])
            any_stream_is_video = any(x['codec_type'] == 'video'
                                      for x in streams if 'codec_type' in x)
            widths = [x['width'] for x in streams if 'width' in x]
            heights = [x['height'] for x in streams if 'height' in x]
            bitrate = program.get('tags', {}).get('variant_bitrate')
            if bitrate:
                bitrate = int(bitrate) / 1000

            pid = program.get('program_id')
            res.append(StreamFlavor(
                media_type='video' if any_stream_is_video else 'audio',
                height=heights[0] if heights else None,
                width=widths[0] if widths else None,
                bitrate=bitrate,
                streams=[
                    HLSBackend(manifest_url, long_probe=True,
                               program_id=pid, is_live=is_live)
                ]
            ))

        return sorted(res, key=lambda x: x.height or 0)



# -*- coding: utf-8 -*-

from utils import fetch_title, fetch_stream_url, fetch_metadata

def test_arkivet_title():
    title = fetch_title('https://svenska.yle.fi/artikel/2014/06/13'
                        '/halla-det-ar-naturvaktarna')

    assert title
    assert title[0].startswith('Seportage om Naturväktarna')


def test_arkivet_stream_url():
    streamurl = fetch_stream_url('https://svenska.yle.fi/artikel/2014/06/13'
                                 '/halla-det-ar-naturvaktarna')
    assert streamurl
    assert '/a.m3u8' in streamurl[0]


def test_arkivet_metadata():
    metadata = fetch_metadata('https://svenska.yle.fi/artikel/2014/06/13'
                              '/halla-det-ar-naturvaktarna')

    assert len(metadata) == 1
    assert metadata[0].get('title').startswith('Seportage om Naturväktarna')
    flavors = metadata[0]['flavors']
    assert all(f.get('media_type') == 'video' for f in flavors)
    assert all('bitrate' in f for f in flavors)
    assert all('height' in f for f in flavors)
    assert all('width' in f for f in flavors)


def test_arkivet_audio_stream_url():
    streamurl = fetch_stream_url(
        'https://svenska.yle.fi/artikel/2014/01/28'
        '/tove-jansson-laser-noveller-ur-dockskapet')

    assert len(streamurl) == 11
    for url in streamurl:
        assert '/a.mp3' in url


def test_arkivet_audio_metadata():
    metadata = fetch_metadata(
        'https://svenska.yle.fi/artikel/2014/01/28'
        '/tove-jansson-laser-noveller-ur-dockskapet')

    assert len(metadata) == 11
    assert metadata[0].get('title').startswith('Apan ur Dockskåpet')
    for m in metadata:
        assert all(f.get('media_type') == 'audio' for f in m.get('flavors'))


def test_arkivet_a__stream_url():
    streamurl = fetch_stream_url('https://svenska.yle.fi/a/7-884297')
    assert streamurl
    assert '/a.m3u8' in streamurl[0]


def test_arkivet_a_metadata():
    metadata = fetch_metadata('https://svenska.yle.fi/a/7-884297')

    assert len(metadata) >= 4
    assert metadata[1].get('title').startswith('Valborg på Borgbacken')
    flavors = metadata[1]['flavors']
    assert all(f.get('media_type') == 'video' for f in flavors)
    assert all('bitrate' in f for f in flavors)
    assert all('height' in f for f in flavors)
    assert all('width' in f for f in flavors)
